@@ -17,7 +17,8 @@ from streamlink.plugin.api.utils import parse_json, parse_query
 from streamlink.stream import (
     HTTPStream, HLSStream, FLVPlaylist, extract_flv_header_tags
 )
-from streamlink.stream.hls import HLSStreamReader, HLSStreamWriter, HLSStreamWorker
+from streamlink.stream.hls import HLSStreamWorker
+from streamlink.stream.hls_filtered import FilteredHLSStreamWriter, FilteredHLSStreamReader
 from streamlink.stream.hls_playlist import M3U8Parser, load as load_hls_playlist
 from streamlink.utils.times import hours_minutes_seconds
 
@@ -119,38 +120,23 @@ _video_schema = validate.Schema(
     }
 )
 
-Segment = namedtuple("Segment", "uri duration title key discontinuity ad byterange date map")
+Segment = namedtuple("Segment", "uri duration title key discontinuity ad byterange date map prefetch")
 
 LOW_LATENCY_MAX_LIVE_EDGE = 2
 
 
 class TwitchM3U8Parser(M3U8Parser):
-    def __init__(self, base_uri=None, disable_ads=False, low_latency=False, **kwargs):
-        super(TwitchM3U8Parser, self).__init__(base_uri, **kwargs)
-        self.disable_ads = disable_ads
-        self.low_latency = low_latency
-        self.has_prefetch_segments = False
-
-    def parse(self, *args):
-        m3u8 = super(TwitchM3U8Parser, self).parse(*args)
-        m3u8.has_prefetch_segments = self.has_prefetch_segments
-
-        return m3u8
-
     def parse_extinf(self, value):
         duration, title = super(TwitchM3U8Parser, self).parse_extinf(value)
-        if title and str(title).startswith("Amazon") and self.disable_ads:
+        if title and str(title).startswith("Amazon"):
             self.state["ad"] = True
 
         return duration, title
 
     def parse_tag_ext_x_twitch_prefetch(self, value):
-        if not self.low_latency:
-            return
-        self.has_prefetch_segments = True
         segments = self.m3u8.segments
         if segments:
-            segments.append(segments[-1]._replace(uri=self.uri(value)))
+            segments.append(segments[-1]._replace(uri=self.uri(value), prefetch=True))
 
     def get_segment(self, uri):
         byterange = self.state.pop("byterange", None)
@@ -170,53 +156,56 @@ class TwitchM3U8Parser(M3U8Parser):
             ad,
             byterange,
             date,
-            map_
+            map_,
+            prefetch=False
         )
 
 
 class TwitchHLSStreamWorker(HLSStreamWorker):
-    def __init__(self, *args, **kwargs):
-        self.playlist_reloads = 0
-        super(TwitchHLSStreamWorker, self).__init__(*args, **kwargs)
+    def __init__(self, reader, *args, **kwargs):
+        self.had_content = False
+        super(TwitchHLSStreamWorker, self).__init__(reader, *args, **kwargs)
 
-    def _reload_playlist(self, text, url):
-        self.playlist_reloads += 1
-        playlist = load_hls_playlist(
-            text,
-            url,
-            parser=TwitchM3U8Parser,
-            disable_ads=self.stream.disable_ads,
-            low_latency=self.stream.low_latency
-        )
-        if (
-            self.stream.disable_ads
-            and self.playlist_reloads == 1
-            and not next((s for s in playlist.segments if not s.ad), False)
-        ):
-            log.info("Waiting for pre-roll ads to finish, be patient")
+    def _reload_playlist(self, *args):
+        return load_hls_playlist(*args, parser=TwitchM3U8Parser)
 
-        return playlist
+    def _playlist_reload_time(self, playlist, sequences):
+        if self.stream.low_latency and sequences:
+            return sequences[-1].segment.duration
 
-    def _set_playlist_reload_time(self, playlist, sequences):
-        if self.stream.low_latency and len(sequences) > 0:
-            self.playlist_reload_time = sequences[-1].segment.duration
-        else:
-            super(TwitchHLSStreamWorker, self)._set_playlist_reload_time(playlist, sequences)
+        return super(TwitchHLSStreamWorker, self)._playlist_reload_time(playlist, sequences)
 
     def process_sequences(self, playlist, sequences):
-        if self.stream.low_latency and self.playlist_reloads == 1 and not playlist.has_prefetch_segments:
-            log.info("This is not a low latency stream")
+        # ignore prefetch segments if not LL streaming
+        if not self.stream.low_latency:
+            sequences = [seq for seq in sequences if not seq.segment.prefetch]
+
+        # check for sequences with real content
+        if not self.had_content:
+            self.had_content = next((True for seq in sequences if not seq.segment.ad), False)
+
+            # When filtering ads, to check whether it's a LL stream, we need to wait for the real content to show up,
+            # since playlists with only ad segments don't contain prefetch segments
+            if (
+                self.stream.low_latency
+                and self.had_content
+                and not next((True for seq in sequences if seq.segment.prefetch), False)
+            ):
+                log.info("This is not a low latency stream")
+
+        # show pre-roll ads message only on the first playlist containing ads
+        if self.stream.disable_ads and self.playlist_sequence == -1 and not self.had_content:
+            log.info("Waiting for pre-roll ads to finish, be patient")
 
         return super(TwitchHLSStreamWorker, self).process_sequences(playlist, sequences)
 
 
-class TwitchHLSStreamWriter(HLSStreamWriter):
-    def write(self, sequence, *args, **kwargs):
-        if not (self.stream.disable_ads and sequence.segment.ad):
-            return super(TwitchHLSStreamWriter, self).write(sequence, *args, **kwargs)
+class TwitchHLSStreamWriter(FilteredHLSStreamWriter):
+    def should_filter_sequence(self, sequence):
+        return self.stream.disable_ads and sequence.segment.ad
 
 
-class TwitchHLSStreamReader(HLSStreamReader):
+class TwitchHLSStreamReader(FilteredHLSStreamReader):
     __worker__ = TwitchHLSStreamWorker
     __writer__ = TwitchHLSStreamWriter
 
@@ -306,9 +295,10 @@ class TwitchAPI(object):
     def call_subdomain(self, subdomain, path, format="json", schema=None, **extra_params):
         subdomain_buffer = self.subdomain
         self.subdomain = subdomain
-        response = self.call(path, format=format, schema=schema, **extra_params)
-        self.subdomain = subdomain_buffer
-        return response
+        try:
+            return self.call(path, format=format, schema=schema, **extra_params)
+        finally:
+            self.subdomain = subdomain_buffer
 
     # Public API calls
 
@@ -327,7 +317,7 @@ class TwitchAPI(object):
     # Private API calls
 
     def access_token(self, endpoint, asset, **params):
-        return self.call("/api/{0}/{1}/access_token".format(endpoint, asset), private=True, **params)
+        return self.call("/api/{0}/{1}/access_token".format(endpoint, asset), private=True, **dict(platform="_", **params))
 
     def hosted_channel(self, **params):
         return self.call_subdomain("tmi", "/hosts", format="", **params)
