@@ -4,7 +4,9 @@ import struct
 
 from collections import defaultdict, namedtuple
 from Crypto.Cipher import AES
+from requests.exceptions import ChunkedEncodingError
 
+from streamlink.compat import urlparse, str
 from streamlink.exceptions import StreamError
 from streamlink.stream import hls_playlist
 from streamlink.stream.ffmpegmux import FFMPEGMuxer, MuxedStream
@@ -12,6 +14,7 @@ from streamlink.stream.http import HTTPStream
 from streamlink.stream.segmented import (SegmentedStreamReader,
                                          SegmentedStreamWriter,
                                          SegmentedStreamWorker)
+from streamlink.utils import LazyFormatter
 
 log = logging.getLogger(__name__)
 Sequence = namedtuple("Sequence", "num segment")
@@ -46,6 +49,7 @@ class HLSStreamWriter(SegmentedStreamWriter):
         self.key_data = None
         self.key_uri = None
         self.key_uri_override = options.get("hls-segment-key-uri")
+        self.stream_data = options.get("hls-segment-stream-data")
 
         if self.ignore_names:
             # creates a regex from a list of segment names,
@@ -62,7 +66,18 @@ class HLSStreamWriter(SegmentedStreamWriter):
         if not self.key_uri_override and not key.uri:
             raise StreamError("Missing URI to decryption key")
 
-        key_uri = self.key_uri_override if self.key_uri_override else key.uri
+        if self.key_uri_override:
+            p = urlparse(key.uri)
+            key_uri = LazyFormatter.format(
+                self.key_uri_override,
+                url=key.uri,
+                scheme=p.scheme,
+                netloc=p.netloc,
+                path=p.path,
+                query=p.query,
+            )
+        else:
+            key_uri = key.uri
 
         if self.key_uri != key_uri:
             res = self.session.http.get(key_uri, exception=StreamError,
@@ -109,6 +124,8 @@ class HLSStreamWriter(SegmentedStreamWriter):
                 return
 
             return self.session.http.get(sequence.segment.uri,
+                                         stream=(self.stream_data
+                                                 and not sequence.segment.key),
                                          timeout=self.timeout,
                                          exception=StreamError,
                                          retries=self.retries,
@@ -139,8 +156,13 @@ class HLSStreamWriter(SegmentedStreamWriter):
 
             self.reader.buffer.write(pkcs7_decode(decrypted_chunk))
         else:
-            for chunk in res.iter_content(chunk_size):
-                self.reader.buffer.write(chunk)
+            try:
+                for chunk in res.iter_content(chunk_size):
+                    self.reader.buffer.write(chunk)
+            except ChunkedEncodingError:
+                log.error("Download of segment {0} failed", sequence.num)
+
+                return
 
         log.debug("Download of segment {0} complete", sequence.num)
 
@@ -155,13 +177,19 @@ class HLSStreamWorker(SegmentedStreamWorker):
         self.playlist_sequence = -1
         self.playlist_sequences = []
         self.playlist_reload_time = 15
-        self.live_edge = self.session.options.get("hls-live-edge")
+        self.playlist_reload_time_override = self.session.options.get("hls-playlist-reload-time")
         self.playlist_reload_retries = self.session.options.get("hls-playlist-reload-attempts")
+        self.live_edge = self.session.options.get("hls-live-edge")
         self.duration_offset_start = int(self.stream.start_offset + (self.session.options.get("hls-start-offset") or 0))
         self.duration_limit = self.stream.duration or (
             int(self.session.options.get("hls-duration")) if self.session.options.get("hls-duration") else None)
         self.hls_live_restart = self.stream.force_restart or self.session.options.get("hls-live-restart")
 
+        if str(self.playlist_reload_time_override).isnumeric() and float(self.playlist_reload_time_override) >= 2:
+            self.playlist_reload_time_override = float(self.playlist_reload_time_override)
+        elif self.playlist_reload_time_override not in ["segment", "live-edge"]:
+            self.playlist_reload_time_override = 0
+
         self.reload_playlist()
 
         if self.playlist_end is None:
@@ -210,19 +238,32 @@ class HLSStreamWorker(SegmentedStreamWorker):
         sequences = [Sequence(media_sequence + i, s)
                      for i, s in enumerate(playlist.segments)]
 
+        self.playlist_reload_time = self._playlist_reload_time(playlist, sequences)
+
         if sequences:
             self.process_sequences(playlist, sequences)
 
+    def _playlist_reload_time(self, playlist, sequences):
+        if self.playlist_reload_time_override == "segment" and sequences:
+            return sequences[-1].segment.duration
+        if self.playlist_reload_time_override == "live-edge" and sequences:
+            return sum([s.segment.duration for s in sequences[-max(1, self.live_edge - 1):]])
+        if self.playlist_reload_time_override > 0:
+            return self.playlist_reload_time_override
+        if playlist.target_duration:
+            return playlist.target_duration
+        if sequences:
+            return sum([s.segment.duration for s in sequences[-max(1, self.live_edge - 1):]])
+
+        return self.playlist_reload_time
+
     def process_sequences(self, playlist, sequences):
         first_sequence, last_sequence = sequences[0], sequences[-1]
 
         if first_sequence.segment.key and first_sequence.segment.key.method != "NONE":
             log.debug("Segments in this playlist are encrypted")
 
-        self.playlist_changed = ([s.num for s in self.playlist_sequences] !=
-                                 [s.num for s in sequences])
-        self.playlist_reload_time = (playlist.target_duration or
-                                     last_sequence.segment.duration)
+        self.playlist_changed = ([s.num for s in self.playlist_sequences] != [s.num for s in sequences])
         self.playlist_sequences = sequences
 
         if not self.playlist_changed:
@@ -354,6 +395,10 @@ class HLSStream(HTTPStream):
 
         return reader
 
+    @classmethod
+    def _get_variant_playlist(cls, res):
+        return hls_playlist.load(res.text, base_uri=res.url)
+
     @classmethod
     def parse_variant_playlist(cls, session_, url, name_key="name",
                                name_prefix="", check_streams=False,
@@ -380,7 +425,7 @@ class HLSStream(HTTPStream):
         res = session_.http.get(url, exception=IOError, **request_params)
 
         try:
-            parser = hls_playlist.load(res.text, base_uri=res.url)
+            parser = cls._get_variant_playlist(res)
         except ValueError as err:
             raise IOError("Failed to parse playlist: {0}".format(err))
 
@@ -410,14 +455,13 @@ class HLSStream(HTTPStream):
                     default_audio = [media]
 
                 # select the first audio stream that matches the users explict language selection
-                if (('*' in audio_select or media.language in audio_select or media.name in audio_select) or
-                        ((not preferred_audio or media.default) and locale.explicit and locale.equivalent(
+                if (('*' in audio_select or media.language in audio_select or media.name in audio_select)
+                        or ((not preferred_audio or media.default) and locale.explicit and locale.equivalent(
                             language=media.language))):
                     preferred_audio.append(media)
 
             # final fallback on the first audio stream listed
-            fallback_audio = fallback_audio or (len(audio_streams) and
-                                                audio_streams[0].uri and [audio_streams[0]])
+            fallback_audio = fallback_audio or (len(audio_streams) and audio_streams[0].uri and [audio_streams[0]])
 
             if playlist.stream_info.resolution:
                 width, height = playlist.stream_info.resolution
@@ -434,11 +478,18 @@ class HLSStream(HTTPStream):
             if name_fmt:
                 stream_name = name_fmt.format(**names)
             else:
-                stream_name = (names.get(name_key) or names.get("name") or
-                               names.get("pixels") or names.get("bitrate"))
+                stream_name = (
+                    names.get(name_key)
+                    or names.get("name")
+                    or names.get("pixels")
+                    or names.get("bitrate")
+                )
 
             if not stream_name:
                 continue
+            if name_prefix:
+                stream_name = "{0}{1}".format(name_prefix, stream_name)
+
             if stream_name in streams:  # rename duplicate streams
                 stream_name = "{0}_alt".format(stream_name)
                 num_alts = len(list(filter(lambda n: n.startswith(stream_name), streams.keys())))
@@ -460,12 +511,12 @@ class HLSStream(HTTPStream):
             external_audio = preferred_audio or default_audio or fallback_audio
 
             if external_audio and FFMPEGMuxer.is_usable(session_):
-                external_audio_msg = ", ".join([
-                    "(language={0}, name={1})".format(x.language, (x.name or "N/A"))
+                external_audio_msg = u", ".join([
+                    u"(language={0}, name={1})".format(x.language, (x.name or "N/A"))
                     for x in external_audio
                 ])
-                log.debug("Using external audio tracks for stream {0} {1}", name_prefix + stream_name,
-                          external_audio_msg)
+                log.debug(u"Using external audio tracks for stream {0} {1}".format(
+                          stream_name, external_audio_msg))
 
                 stream = MuxedHLSStream(session_,
                                         video=playlist.uri,
@@ -481,6 +532,6 @@ class HLSStream(HTTPStream):
                              start_offset=start_offset,
                              duration=duration,
                              **request_params)
-            streams[name_prefix + stream_name] = stream
+            streams[stream_name] = stream
 
         return streams
